==> generator-xx-0.0.19.tgz: package/generators/app/templates/gulpfile.js <==
var pkg = require('./package.json');
var fs = require('fs');
var $argv = require('yargs').argv;
var $path = require('path');
var $gulp = require('gulp');
var $postcss = require('gulp-postcss');
var $autoprefixer = require('autoprefixer');
var $fileinclude = require('gulp-file-include');
var $gulpif = require('gulp-if');
var $del = require('del');
var $cssmin = require('gulp-cssmin');
var $webpack = require('gulp-webpack');
var $uglify = require('gulp-uglify');
var $replace = require('gulp-replace');
var $nproxy = require('gulp-nproxy');
<% if (tnpm) { %>
var $upload = require('@tencent/gulp-tcms-upload');
<% } %>
var $sprite = require('gulp.spritesmith');
var $iconfont = require('gulp-iconfont');
var $iconfontCss = require('gulp-iconfont-css');
var $ExtractTextPlugin = require('extract-text-webpack-plugin');
var $rd = require('rd');
var $rev = require('gulp-rev');
var $revCssUrl = require('gulp-rev-css-url');
var $less = require('gulp-less');
var $cssimport = require("gulp-cssimport");

var $CommonsChunkPlugin = require("webpack/lib/optimize/CommonsChunkPlugin");
var $httpush = require('httpush');

var cookie = '';//upload cookie
var prefix = '';// 制作稿上传之后的cdn前缀，可直接覆盖，不带版本管理
var php_dir = 'php/htdocs';//PHP目录
var static_dir = php_dir + '/static';//静态资源存放目录
var manifest_dir = php_dir + '/views/ver.json';//版本控制文件位置
var php_views_dir = php_dir + '/views';// views目录
var php_views_svn_dir = php_dir + '/svn/views';// svn上php工程的views目录

// css sprite clean, iconfont clean
$gulp.task('style:clean', function(){
  $del.sync(['./css/sprite/', './css/inc/', './css/fonts/']);
});

//css处理
// 1.雪碧图处理，并移动到html目录
// 2.font处理，并移动到html目录
// 3.css处理，并移动到html目录
// $gulp.task('css:dw', ['sprite', 'iconfont'], function(){
//   return $gulp.src('css/*.css')
//     .pipe($fileinclude())
//     .pipe($postcss([
//       $autoprefixer
//     ]))
//     .pipe($replace(/\.\.\/sprite/g, 'sprite'))//修改雪碧图路径
//     .pipe($cssmin())
//     .pipe($gulp.dest(static_dir + '/css/'));
// });

// less处理
$gulp.task('less:dw', ['sprite', 'iconfont'], function(){
  return $gulp.src('*.less', {
      cwd : './css/',
      base : './css/'
    })
    .pipe($less())
    .pipe($cssimport())
    .pipe($postcss([
      $autoprefixer
    ]))
    .pipe($replace(/\.\.\/sprite/g, 'sprite'))//修改雪碧图路径
    .pipe($cssmin())
    .pipe($gulp.dest(static_dir + '/css/'));
});

$gulp.task('_font:icon', function(){
  var fontName = 'iconfont';
  return $gulp.src(['css/slices/*.svg'])
    .pipe($iconfontCss({
      fontName : fontName,
      fontPath : 'fonts/',
      targetPath : '../inc/iconfont.css'
    }))
    .pipe($iconfont({
      fontName : fontName,
      formats: ['ttf', 'eot', 'woff', 'svg', 'woff2'],
      timestamp: 0
    }))
    .pipe($gulp.dest('css/fonts/'));
});

// font处理 将font移动到html目录
// 将css最小化之后，font才能正常工作
$gulp.task('iconfont', ['_font:icon'], function(){
  return $gulp.src('css/fonts/*.*')
    .pipe($gulp.dest(static_dir + '/css/fonts/'));
});

//css sprite处理
$gulp.task('_css:sprite', function(){
  return $gulp.src('css/slices/*.png')
  .pipe($sprite({
    imgName : 'sprite/image.png',
    cssName : 'inc/sprite.css'
  }))
  .pipe($gulp.dest('css/'));
});

// sprite处理 将sprite移动到html目录
$gulp.task('sprite', ['_css:sprite'], function(){
  return $gulp.src('css/sprite/*.*')
    .pipe($gulp.dest(static_dir + '/css/sprite/'));
});

$gulp.task('build:css', ['style:clean', 'less:dw'], function(){

});

//js webpack
function getEntry(dir){
  var files = {};
  $rd.eachSync(dir, function(f,s){
    if(s.isFile()){
      var ext = $path.extname(f);
      var file = $path.basename(f).replace(ext, '');
      files[file] = dir + file;
    }
  });
  return files;
}
$gulp.task('js:webpack', function(){
  var dir = './js/entry/';// js目录
  return $gulp.src(dir + '*.js')
    .pipe($webpack({
      module : {
        loaders: [
          {
            test : /\.tpl$/,
            loader : 'html?minimize'
          },
          {
            test : /\.css$/,
            loader : 'html'
          },
          {
            test: /\.(png|jpg|jpeg|gif|woff)$/,
            loader: 'url-loader?limit=8192'
          }
        ]
      },
      entry : getEntry(dir),
      output : {
        filename : '[name].js',
        publicPath: "/"
      },
      plugins: [
          // new $CommonsChunkPlugin("commons.chunk.js") //提取entry中的公共部分
      ]
    }))
    .pipe($gulp.dest(static_dir + '/js/'));
});

$gulp.task('mini:js', function(){
  return $gulp.src('./release/js/**/*')
          .pipe($uglify())
          .pipe($gulp.dest('./release/js/'));
});

$gulp.task('build:js', ['js:webpack'], function(){

});

$gulp.task('build:clean', function(){
  $del.sync([static_dir + '/css/', static_dir + '/js/']);
});

// 将rev-manifest.json里对象的key中'.js'更换成'_js'，配合php中smarty使用
var revTransformer = {
  parse : JSON.parse,
  stringify : function(manifest, replacer, space){
    for(var key in manifest){
      var ext = $path.extname(key);
      var newext = ext.replace('.', '_');
      manifest[key.replace(ext, newext)] = manifest[key];

      if(ext.indexOf('.') != -1){
        delete manifest[key];
      }
    }

    return JSON.stringify(manifest, replacer, space);
  }
}

$gulp.task('release:clean', function(){
  $del.sync(['./release/']);
});

$gulp.task('release:css', ['build:css'], function(){
  return $gulp.src([static_dir + '/css/**/*'])
  .pipe($rev())
  .pipe($revCssUrl())
  .pipe($gulp.dest('./release/css'))
  .pipe($rev.manifest(manifest_dir, {base:'./release', merge:true, transformer:revTransformer}))
  .pipe($gulp.dest('./release'));
});

$gulp.task('release:js', ['build:js'], function(){
  return $gulp.src(static_dir + '/js/**/*')
  .pipe($rev())
  .pipe($gulp.dest('./release/js'))
  .pipe($rev.manifest(manifest_dir, {base:'./release', merge:true, transformer:revTransformer}))
  .pipe($gulp.dest('./release'));
});

$gulp.task('release:svn', ['release:clean', 'release:css', 'release:js'], function(){
  return $gulp.src(php_views_dir + '/**/*')
  .pipe($gulp.dest(php_views_svn_dir));
});

// ===========================npm script start============================

// 项目build
$gulp.task('build',['build:clean', 'build:css', 'build:js'], function(){
  if($argv.w){// watching
    $gulp.watch(['./css/**/*.less', './css/slices/*'], ['build:css']);// css处理

    $gulp.watch(['./js/**/*'], ['build:js']);// js处理
  }
});

$gulp.task('release', ['build:clean' ,'release:svn'], function(){

});

// 目标路径
// console.log(file.getHashRelease()) --> /protected/config/wii_ui/aaa/config/chart-map.json
// 文件内容
// console.log(file.getContent()) --> 
// 上传文件的文件名
// console.log(file.subpath) --> /aaa/config/chart-map.json
function getAllFiles(dir, to){
  var files = [];
  var views_abs_dir = __dirname + '/php/htdocs/views/';

  $rd.eachSync(dir, function(f, s){
    if(s.isFile()){
      var file = {};
      
      file.subpath = f.replace(views_abs_dir, '');

      file.getHashRelease = function(){
        return file.subpath;
      }

      file.getContent = function(){
        var content = fs.readFileSync(f, {encoding : 'utf-8'});
        return content;
      }

      files.push(file);
    }
  });


  return files;
}

// 项目部署
$gulp.task('deploy', ['release'], function(){
  <% if (tnpm) { %>
  // 182
  // 配置host（i.city.qq.com xxx.xxx.xxx.182）
  var receiver = 'http://i.city.qq.com/receiver.php';
  var to = '/data/qqcom_app/i_city/apps/city/views/Bargain';
  <% }else{ %>
  // local
  var receiver = 'http://127.0.0.1:8999/receiver';
  var to = '/Users/yourname/Dev/nodedocs';
  <% } %>

  $httpush({
    receiver : receiver,
    to : to
  }, getAllFiles(php_views_dir, to));
});

<% if (tnpm) { %>
// 项目upload
$gulp.task('upload', ['mini:js'], function(){
  return $gulp.src('**/*',{
    cwd : './release/'
  })
  .pipe($gulpif('*.js', $uglify()))
  .pipe($upload({
    overwrite : false,
    cookie : cookie,
    uploadTo : 'test'
  }));
});

// 上传images
$gulp.task('upload:images', function(){
  return $gulp.src('**/*',{
    cwd : static_dir + '/images/'
  })
  .pipe($upload({
    overwrite : true,
    cookie : cookie,
    uploadTo : 'test/images'
  }));
});
<% } %>

// http autoResponder(port:9998)
$gulp.task('nproxy', function(){
  var dir = '/Users/aslinwang/Dev/jsdevs/city/test';// 绝对路径，按自己的目录情况更改
  // var dir = 'E://jsdevs/city/mpsvote';
  $nproxy({
    debug: false,
    port : 9998,
    rule : [
      {
        pattern : 'http://mat1.gtimg.com/city/test/.*/css/',
        responder : dir + '/dist/css/'
      },
      {
        pattern : 'http://mat1.gtimg.com/city/test/.*/js/',
        responder : dir + '/dist/js/'
      }
    ]
  });
});

// ===========================npm script end==============================

$gulp.task('default', [
  'build'
]);

==> generator-xx-0.0.21.tgz: package/generators/app/templates/gulpfile.js <==
var pkg = require('./package.json');
var fs = require('fs');
var $argv = require('yargs').argv;
var $path = require('path');
var $gulp = require('gulp');
var $postcss = require('gulp-postcss');
var $autoprefixer = require('autoprefixer');
var $fileinclude = require('gulp-file-include');
var $gulpif = require('gulp-if');
var $del = require('del');
var $cssmin = require('gulp-cssmin');
var $webpack = require('gulp-webpack');
var $uglify = require('gulp-uglify');
var $replace = require('gulp-replace');
var $nproxy = require('gulp-nproxy');
var $gutil = require('gulp-util');
var $spawn = require('child_process').spawn;
<% if (tnpm) { %>
var $upload = require('@tencent/gulp-tcms-upload');
<% } %>
var $sprite = require('gulp.spritesmith');
var $iconfont = require('gulp-iconfont');
var $iconfontCss = require('gulp-iconfont-css');
var $ExtractTextPlugin = require('extract-text-webpack-plugin');
var $rd = require('rd');
var $rev = require('gulp-rev');
var $revCssUrl = require('gulp-rev-css-url');
var $less = require('gulp-less');
var $cssimport = require("gulp-cssimport");

var $CommonsChunkPlugin = require("webpack/lib/optimize/CommonsChunkPlugin");
var $httpush = require('httpush');

var pkg = require('./package.json');
var cookie = pkg.config.cookie;//upload cookie
var php_dir = 'php/htdocs';//PHP目录
var static_dir = php_dir + '/static';//静态资源存放目录
var manifest_dir = php_dir + '/views/ver.json';//版本控制文件位置
var php_views_dir = php_dir + '/views';// views目录
var php_svn_dir = php_dir + '/svn';//svn目录
// svn目录按需配置
var php_svn_dir_assets = php_svn_dir + '/assets';// svn上php工程的assets目录,用于静态资源直接存放在服务器而不是CDN的情况
var php_svn_dir_views = php_svn_dir + '/views';// svn上php工程的views目录

// css sprite clean, iconfont clean
$gulp.task('style:clean', function(){
  $del.sync(['./css/sprite/', './css/inc/', './css/fonts/']);
});

//css处理
// 1.雪碧图处理，并移动到html目录
// 2.font处理，并移动到html目录
// 3.css处理，并移动到html目录
// $gulp.task('css:dw', ['sprite', 'iconfont'], function(){
//   return $gulp.src('css/*.css')
//     .pipe($fileinclude())
//     .pipe($postcss([
//       $autoprefixer
//     ]))
//     .pipe($replace(/\.\.\/sprite/g, 'sprite'))//修改雪碧图路径
//     .pipe($cssmin())
//     .pipe($gulp.dest(static_dir + '/css/'));
// });

// less处理
$gulp.task('less:dw', ['sprite', 'iconfont'], function(){
  return $gulp.src('*.less', {
      cwd : './css/',
      base : './css/'
    })
    .pipe($less())
    .pipe($cssimport())
    .pipe($postcss([
      $autoprefixer
    ]))
    .pipe($replace(/\.\.\/sprite/g, 'sprite'))//修改雪碧图路径
    .pipe($cssmin())
    .pipe($gulp.dest(static_dir + '/css/'));
});

$gulp.task('_font:icon', function(){
  var fontName = 'iconfont';
  return $gulp.src(['css/slices/*.svg'])
    .pipe($iconfontCss({
      fontName : fontName,
      fontPath : 'fonts/',
      targetPath : '../inc/iconfont.css'
    }))
    .pipe($iconfont({
      fontName : fontName,
      formats: ['ttf', 'eot', 'woff', 'svg', 'woff2'],
      timestamp: 0
    }))
    .pipe($gulp.dest('css/fonts/'));
});

// font处理 将font移动到html目录
// 将css最小化之后，font才能正常工作
$gulp.task('iconfont', ['_font:icon'], function(){
  return $gulp.src('css/fonts/*.*')
    .pipe($gulp.dest(static_dir + '/css/fonts/'));
});

//css sprite处理
$gulp.task('_css:sprite', function(){
  return $gulp.src('css/slices/*.png')
  .pipe($sprite({
    imgName : 'sprite/image.png',
    cssName : 'inc/sprite.css'
  }))
  .pipe($gulp.dest('css/'));
});

// sprite处理 将sprite移动到html目录
$gulp.task('sprite', ['_css:sprite'], function(){
  return $gulp.src('css/sprite/*.*')
    .pipe($gulp.dest(static_dir + '/css/sprite/'));
});

$gulp.task('build:css', ['style:clean', 'less:dw'], function(){

});

//js webpack
function getEntry(dir){
  var files = {};
  $rd.eachSync(dir, function(f,s){
    if(s.isFile()){
      var ext = $path.extname(f);
      var file = $path.basename(f).replace(ext, '');
      files[file] = dir + file;
    }
  });
  return files;
}
$gulp.task('js:webpack', function(){
  var dir = './js/entry/';// js目录
  return $gulp.src(dir + '*.js')
    .pipe($webpack({
      module : {
        loaders: [
          {
            test : /\.tpl$/,
            loader : 'html?minimize'
          },
          {
            test : /\.css$/,
            loader : 'html'
          },
          {
            test: /\.(png|jpg|jpeg|gif|woff)$/,
            loader: 'url-loader?limit=8192'
          }
        ]
      },
      entry : getEntry(dir),
      output : {
        filename : '[name].js',
        publicPath: "/"
      },
      plugins: [
          // new $CommonsChunkPlugin("commons.chunk.js") //提取entry中的公共部分
      ]
    }))
    .pipe($gulp.dest(static_dir + '/js/'));
});

$gulp.task('mini:js', function(){
  return $gulp.src('./release/js/**/*')
          .pipe($uglify())
          .pipe($gulp.dest('./release/js/'));
});

$gulp.task('build:js', ['js:webpack'], function(){

});

$gulp.task('build:clean', function(){
  $del.sync([static_dir + '/css/', static_dir + '/js/']);
});

// 将rev-manifest.json里对象的key中'.js'更换成'_js'，配合php中smarty使用
var revTransformer = {
  parse : JSON.parse,
  stringify : function(manifest, replacer, space){
    for(var key in manifest){
      var ext = $path.extname(key);
      var newext = ext.replace('.', '_');
      manifest[key.replace(ext, newext)] = manifest[key];

      if(ext.indexOf('.') != -1){
        delete manifest[key];
      }
    }

    return JSON.stringify(manifest, replacer, space);
  }
}

$gulp.task('release:clean', function(){
  $del.sync(['./release/']);
});

$gulp.task('release:css', ['build:css'], function(){
  return $gulp.src([static_dir + '/css/**/*'])
  .pipe($rev())
  .pipe($revCssUrl())
  .pipe($gulp.dest('./release/css'))
  .pipe($rev.manifest(manifest_dir, {base:'./release', merge:true, transformer:revTransformer}))
  .pipe($gulp.dest('./release'));
});

$gulp.task('release:js', ['build:js'], function(){
  return $gulp.src(static_dir + '/js/**/*')
  .pipe($rev())
  .pipe($gulp.dest('./release/js'))
  .pipe($rev.manifest(manifest_dir, {base:'./release', merge:true, transformer:revTransformer}))
  .pipe($gulp.dest('./release'));
});

$gulp.task('release:img', function(){
  return $gulp.src(static_dir + '/images/**/*')//将图片资源也发布
  .pipe($gulp.dest('./release/images'));
});

$gulp.task('release:svn', ['release:clean', 'release:css', 'release:js'<%if(!cdn){%>, 'release:img'<%}%>], function(){
  <%if(!cdn){%>
    return $gulp.src('release/**/*')
    .pipe($gulpif('*.js', $uglify()))
    .pipe($gulp.dest(php_svn_dir_assets));
  <%}else{%>
    return $gulp.src(php_views_dir + '/**/*')
    .pipe($gulp.dest(php_svn_dir_views));
  <%}%>
});

// ===========================npm script start============================

// 项目build with watch
$gulp.task('build:w',['build:clean', 'build:css', 'build:js'], function(){
  // if($argv.w){// watching
    $gulp.watch(['./css/**/*.less', './css/slices/*'], ['build:css']);// css处理
    $gulp.watch(['./js/**/*'], ['build:js']);// js处理
  // }
});
// 项目build
// $gulp.task('build',['build:clean', 'build:css', 'build:js'], function(){
  
// });

$gulp.task('release', ['build:clean' ,'release:svn'], function(){

});

// 目标路径
// console.log(file.getHashRelease()) --> /protected/config/wii_ui/aaa/config/chart-map.json
// 文件内容
// console.log(file.getContent()) --> 
// 上传文件的文件名
// console.log(file.subpath) --> /aaa/config/chart-map.json
function getAllFiles(dir, to){
  var files = [];
  var views_abs_dir = __dirname + '/php/htdocs/views/';

  $rd.eachSync(dir, function(f, s){
    if(s.isFile() && f.indexOf('.svn') == -1){// 非.svn目录中的文件
      var file = {};
      
      file.subpath = f.replace(views_abs_dir, '');

      file.getHashRelease = function(){
        return file.subpath;
      }

      file.getContent = function(){
        var content = fs.readFileSync(f, {encoding : 'utf-8'});
        return content;
      }

      files.push(file);
    }
  });


  return files;
}

// 项目部署
$gulp.task('deploy', ['release'], function(){
  <% if (tnpm) { %>
  // 182
  // 配置host（receiver.qq.com xxx.xxx.xxx.182）
  var receiver = 'http://receiver.qq.com/receiver.php';
  var to = '/data/qqcom_app/i_city/apps/city/views/Bargain';
  <% }else{ %>
  // local
  var receiver = 'http://127.0.0.1:8999/receiver';
  var to = '/Users/yourname/Dev/nodedocs';
  <% } %>

  $httpush({
    receiver : receiver,
    to : to
  }, getAllFiles(php_views_dir, to));
});

// 启动server,同时监听文件改动并编译
$gulp.task('server', ['build:w'], function(cb){
  var host = pkg.config.devHost;
  var port = pkg.config.devPort;
  var child = $spawn("sudo", ['php', '-S' + host + ':' + port + ' ', '-t./php/htdocs'], {cwd: process.cwd()});

  <% if (mac) { %>
  if(port > 1024){// 开1024以上的端口不需要sudo
    child = $spawn("php", ['-S' + host + ':' + port + ' ', '-t./php/htdocs'], {cwd: process.cwd()});
  }
  <% } %>

  $gutil.log('server started. visit http://' + host + ':' + port);
  child.stdout.setEncoding('utf8');
  child.stdout.on('data', function (data) {
      console.log(data);
  });

  child.stderr.setEncoding('utf8');
  child.stderr.on('data', function (data) {
    console.log($gutil.colors.green(data));
  });

  child.on('close', function(code) {
    $gutil.log("Done with exit code", code);
    $gutil.log("You access complete stdout and stderr from here"); // stdout, stderr
  });
});
<% if (tnpm) { %>
// 项目upload
$gulp.task('upload', ['mini:js'], function(){
  return $gulp.src('**/*',{
    cwd : './release/'
  })
  .pipe($gulpif('*.js', $uglify()))
  .pipe($upload({
    overwrite : false,
    cookie : cookie,
    uploadTo : 'test'
  }));
});

// 上传images
$gulp.task('upload:images', function(){
  return $gulp.src('**/*',{
    cwd : static_dir + '/images/'
  })
  .pipe($upload({
    overwrite : true,
    cookie : cookie,
    uploadTo : 'test/images'
  }));
});
<% } %>

// http autoResponder(port:9998)
$gulp.task('nproxy', function(){
  var dir = '/Users/aslinwang/Dev/jsdevs/city/test';// 绝对路径，按自己的目录情况更改
  // var dir = 'E://jsdevs/city/mpsvote';
  $nproxy({
    debug: false,
    port : 9998,
    rule : [
      {
        pattern : 'http://mat1.gtimg.com/city/test/.*/css/',
        responder : dir + '/dist/css/'
      },
      {
        pattern : 'http://mat1.gtimg.com/city/test/.*/js/',
        responder : dir + '/dist/js/'
      }
    ]
  });
});

// ===========================npm script end==============================

$gulp.task('default', [
  'server'
]);
